// Copyright (c) 2015 Douglas Otwell
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
#include <pebble.h>
#include "main_menu.h"
#include "common.h"
#include "basic_menu_layer.h"
#include "player.h"
#include "remote.h"
#include "goto.h"
#include "playlists.h"
#include "addons.h"
#include "power.h"

#define NUM_WINDOWS 6

static Window *s_window;
static MenuLayer *s_menu;
static GBitmap *s_icon_player;
static GBitmap *s_icon_remote;
static GBitmap *s_icon_goto;
static GBitmap *s_icon_list;
static GBitmap *s_icon_addons;
static GBitmap *s_icon_power;
static BasicMenuModel *s_model;
static MenuLayer *s_menu_layer;

static uint16_t get_num_rows_callback(MenuLayer *menu_layer, uint16_t section_index, void *context) {
  return NUM_WINDOWS;
}

static int16_t get_cell_height_callback(struct MenuLayer *menu_layer, MenuIndex *cell_index, void *context) {
  return CHECKBOX_WINDOW_CELL_HEIGHT;
}

static void draw_row_callback(GContext *ctx, Layer *cell_layer, MenuIndex *cell_index, void *context) {
  switch(cell_index->row) {
    case 0: menu_cell_basic_draw(ctx, cell_layer, "Player", NULL, s_icon_player);   break;
    case 1: menu_cell_basic_draw(ctx, cell_layer, "Remote", NULL, s_icon_remote);   break;
    case 2: menu_cell_basic_draw(ctx, cell_layer, "Go to", NULL, s_icon_goto);      break;
    case 3: menu_cell_basic_draw(ctx, cell_layer, "Playlists", NULL, s_icon_list);  break;
    case 4: menu_cell_basic_draw(ctx, cell_layer, "Addons", NULL, s_icon_addons);   break;
    case 5: menu_cell_basic_draw(ctx, cell_layer, "Power", NULL, s_icon_power);     break;
    default:                                                                        break;
  }
}

static void select_callback(struct MenuLayer *menu_layer, MenuIndex *cell_index, void *context) {
  switch(cell_index->row) {
    case 0:  show_player();    break;
    case 1:  show_remote();    break;
    case 2:  show_goto();      break;
    case 3:  show_playlists(); break;
    case 4:  show_addons();    break;
    case 5:  show_power();     break;
    default:                   break;
  }
}

static void draw_header_callback(GContext *ctx, const Layer *cell_layer, uint16_t section_index, void *context) {
  menu_cell_basic_header_draw(ctx, cell_layer, "Rockodi - Kodi Remote");
}

static void menu_select_cb(MenuLayer *menu_layer, MenuIndex *cell_index, BasicMenuItem *item) {
  void (*callback)(void) = item->value;
  callback();
}

static int16_t get_header_height_callback(struct MenuLayer *menu_layer, uint16_t section_index, void *context) {
  return MENU_CELL_BASIC_HEADER_HEIGHT;
}

static uint16_t get_num_sections_callback(struct MenuLayer *menu_layer, void *context) {
  return 1;
}

static void window_load(Window *window) {
  Layer *window_layer = window_get_root_layer(window);
  GRect bounds = layer_get_bounds(window_layer);

  s_icon_player = gbitmap_create_with_resource(RESOURCE_ID_ICON_PLAYER_ALPHA);
  s_icon_remote = gbitmap_create_with_resource(RESOURCE_ID_ICON_REMOTE);
  s_icon_goto = gbitmap_create_with_resource(RESOURCE_ID_ICON_GOTO);
  s_icon_list = gbitmap_create_with_resource(RESOURCE_ID_ICON_LIST);
  s_icon_addons = gbitmap_create_with_resource(RESOURCE_ID_ICON_ADDONS);
  s_icon_power = gbitmap_create_with_resource(RESOURCE_ID_ICON_POWER);

  s_menu_layer = menu_layer_create(bounds);
  menu_layer_set_click_config_onto_window(s_menu_layer, window);
  #ifdef PBL_PLATFORM_BASALT
  menu_layer_set_normal_colors(s_menu_layer, GColorBlack, GColorWhite);
  menu_layer_set_highlight_colors(s_menu_layer, GColorCobaltBlue, GColorWhite);
  #endif
  menu_layer_set_callbacks(s_menu_layer, NULL, (MenuLayerCallbacks) {
      .get_num_rows = (MenuLayerGetNumberOfRowsInSectionsCallback)get_num_rows_callback,
      .draw_row = (MenuLayerDrawRowCallback)draw_row_callback,
      .get_cell_height = (MenuLayerGetCellHeightCallback)get_cell_height_callback,
      .select_click = (MenuLayerSelectCallback)select_callback,
      .draw_header = (MenuLayerDrawHeaderCallback)draw_header_callback,
      .get_header_height = (MenuLayerGetHeaderHeightCallback)get_header_height_callback,
      .get_num_sections = (MenuLayerGetNumberOfSectionsCallback)get_num_sections_callback,
  });
  layer_add_child(window_layer, menu_layer_get_layer(s_menu_layer));
}

static void window_unload(Window* window) {
  window_destroy(s_window);
  gbitmap_destroy(s_icon_player);
  gbitmap_destroy(s_icon_remote);
  gbitmap_destroy(s_icon_goto);
  gbitmap_destroy(s_icon_list);
  gbitmap_destroy(s_icon_addons);
  gbitmap_destroy(s_icon_power);
  basic_menu_layer_destroy(s_menu);
  basic_menu_model_destroy(s_model);
}

void show_main_menu(void) {
  s_window = window_create();
  window_set_window_handlers(s_window, (WindowHandlers) {
    .load = window_load,
    .unload = window_unload,
  });
  window_stack_push(s_window, true);
}

void hide_main_menu(void) {
  window_stack_remove(s_window, true);
}